"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Broker = void 0;
const aedes_1 = __importDefault(require("aedes"));
const net_1 = __importDefault(require("net"));
const client_1 = require("./client");
const buffer_1 = require("buffer");
class Broker {
    constructor(port) {
        this.CONFIG_TOPIC = 'initial';
        this.aedes = aedes_1.default({ authorizeForward: this.forwardHandler.bind(this), authorizePublish: this.decodeMessage.bind(this) });
        this.clients = [];
        this.server = net_1.default.createServer(this.aedes.handle);
        this.server.listen(port, () => console.log(`S-MQTT server starts on port: ${port}`));
        this.aedes.on('client', this.prepareUser.bind(this));
        this.aedes.on('subscribe', this.reloadSubscribtion.bind(this));
    }
    decodeMessage(client, data, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            if (client) {
                try {
                    const inputJson = JSON.parse(data.payload);
                    inputJson.iv = buffer_1.Buffer.from(inputJson.iv);
                    inputJson.ephemPublicKey = buffer_1.Buffer.from(inputJson.ephemPublicKey);
                    inputJson.ciphertext = buffer_1.Buffer.from(inputJson.ciphertext);
                    inputJson.mac = buffer_1.Buffer.from(inputJson.mac);
                    const user = this.findUser(client);
                    const decrypted = yield user.decryptInput(inputJson);
                    data.payload = decrypted;
                    console.log('message:' + decrypted);
                }
                catch (_a) {
                    console.log('[INFO] Invalid syntax from clientId: ' + client.id + ' data: ' + data.payload);
                }
                callback();
            }
        });
    }
    forwardHandler(client, data) {
        if (data.topic !== this.CONFIG_TOPIC) {
            const newData = Object.assign({}, data);
            const secureClient = this.findUser(client);
            secureClient.encryptOutput(data.payload).then(ecies => {
                newData.payload = buffer_1.Buffer.from(JSON.stringify(ecies));
            });
            return newData;
        }
        else {
            return data;
        }
    }
    findUser(client) {
        for (const c of this.clients) {
            if (c.baseClient === client) {
                return c;
            }
        }
        return null;
    }
    prepareUser(client) {
        const sclient = new client_1.SecureClient(client);
        this.clients.push(sclient);
        const cmd = 'publish';
        const qos = 0;
        const keys = { outputPrivateKey: sclient.outputPrivateKey, inputPublicKey: sclient.inputPublicKey };
        const packet = {
            qos,
            retain: true,
            topic: 'initial',
            payload: JSON.stringify(keys),
            dup: false,
            cmd
        };
        client.publish(packet, () => console.log(`Przywitano ${sclient.baseClient.id}`));
    }
    reloadSubscribtion(sub, client) {
        for (const t of this.clients) {
            if (t.baseClient === client) {
                t.subscription = sub;
            }
        }
    }
}
exports.Broker = Broker;
//# sourceMappingURL=broker.js.map